/******************************************************************
   IP*Works! ZIP V9 C++ Edition
   Copyright (c) 2012 /n software inc. - All rights reserved.
*******************************************************************/

#ifndef _IPWORKSZIP_BZIP2_H_
#define _IPWORKSZIP_BZIP2_H_

#define IPWORKSZIP_ONLY_TYPES
#include "ipworkszip.h"
#include "ipworkszip.key"


extern "C" void* IPWORKSZIP_CALL IPWorksZip_Bzip2_Create(PIPWORKSZIP_CALLBACK lpSink, void *lpContext, char *lpOemKey);
extern "C" int   IPWORKSZIP_CALL IPWorksZip_Bzip2_Destroy(void *lpObj);
extern "C" int   IPWORKSZIP_CALL IPWorksZip_Bzip2_CheckIndex(void *lpObj, int propid, int arridx);
extern "C" void* IPWORKSZIP_CALL IPWorksZip_Bzip2_Get(void *lpObj, int propid, int arridx, int *lpcbVal);
extern "C" int   IPWORKSZIP_CALL IPWorksZip_Bzip2_Set(void *lpObj, int propid, int arridx, const void *val, int cbVal);
extern "C" int   IPWORKSZIP_CALL IPWorksZip_Bzip2_Do(void *lpObj, int methid, int cparam, void *param[], int cbparam[]);
extern "C" char* IPWORKSZIP_CALL IPWorksZip_Bzip2_GetLastError(void *lpObj);
extern "C" int   IPWORKSZIP_CALL IPWorksZip_Bzip2_GetLastErrorCode(void *lpObj);
extern "C" int   IPWORKSZIP_CALL IPWorksZip_Bzip2_StaticInit(void *hInst);

#ifdef WIN32
#include <windows.h>
#pragma warning(disable:4311) 
#pragma warning(disable:4312) 
#endif

typedef struct {
  int Index;
  int Skip;
  int reserved;
} Bzip2BeginFileEventParams;

typedef struct {
  int Index;
  int reserved;
} Bzip2EndFileEventParams;

typedef struct {
  const char* Description;
  int ErrorCode;
  int Index;
  const char* Filename;
  int Ignore;
  int reserved;
} Bzip2ErrorEventParams;

typedef struct {
  const char* Filename;
  int Overwrite;
  int reserved;
} Bzip2OverwriteEventParams;

typedef struct {
  const char* Data;
  const char* Filename;
  ns_int64 *pBytesProcessed;
  int PercentProcessed;
  int lenData;
  int reserved;
} Bzip2ProgressEventParams;



class Bzip2 {
  
  public: //events
  
    virtual int FireBeginFile(Bzip2BeginFileEventParams *e) {return 0;}
    virtual int FireEndFile(Bzip2EndFileEventParams *e) {return 0;}
    virtual int FireError(Bzip2ErrorEventParams *e) {return 0;}
    virtual int FireOverwrite(Bzip2OverwriteEventParams *e) {return 0;}
    virtual int FireProgress(Bzip2ProgressEventParams *e) {return 0;}


  protected:

    void *m_pObj;
    
    static int IPWORKSZIP_CALL Bzip2EventSink(void *lpObj, int event_id, int cparam, void *param[], int cbparam[]) {
      int ret_code = 0;
      if (event_id > 10000) return ((Bzip2*)lpObj)->Bzip2EventSinkW(event_id - 10000, cparam, param, cbparam);
      switch (event_id) {
         case 1: {
            Bzip2BeginFileEventParams e = {(int)IPZ64CAST(param[0]), (int)IPZ64CAST(param[1]),  0};
            ret_code = ((Bzip2*)lpObj)->FireBeginFile(&e);
            param[1] = (void*)IPZ64CAST(e.Skip);
            break;
         }
         case 2: {
            Bzip2EndFileEventParams e = {(int)IPZ64CAST(param[0]),  0};
            ret_code = ((Bzip2*)lpObj)->FireEndFile(&e);
            break;
         }
         case 3: {
            Bzip2ErrorEventParams e = {(char*)IPZ64CAST(param[0]), (int)IPZ64CAST(param[1]), (int)IPZ64CAST(param[2]), (char*)IPZ64CAST(param[3]), (int)IPZ64CAST(param[4]),  0};
            ret_code = ((Bzip2*)lpObj)->FireError(&e);
            param[4] = (void*)IPZ64CAST(e.Ignore);
            break;
         }
         case 4: {
            Bzip2OverwriteEventParams e = {(char*)IPZ64CAST(param[0]), (int)IPZ64CAST(param[1]),  0};
            ret_code = ((Bzip2*)lpObj)->FireOverwrite(&e);
            param[0] = (void*)IPZ64CAST(e.Filename);
            param[1] = (void*)IPZ64CAST(e.Overwrite);
            break;
         }
         case 5: {
            Bzip2ProgressEventParams e = {(char*)IPZ64CAST(param[0]), (char*)IPZ64CAST(param[1]), (ns_int64*)IPZ64CAST(param[2]), (int)IPZ64CAST(param[3]), (int)IPZ64CAST(cbparam[0]),  0};
            ret_code = ((Bzip2*)lpObj)->FireProgress(&e);
            break;
         }

      }
      return ret_code;
    }

    virtual int Bzip2EventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {return 0;}

  public:

    Bzip2(char *lpOemKey = (char*)IPWORKSZIP_OEMKEY_36) {
      m_pObj = IPWorksZip_Bzip2_Create(Bzip2EventSink, (void*)this, (char*)lpOemKey);
    }

    virtual ~Bzip2() {
      IPWorksZip_Bzip2_Destroy(m_pObj);
    }

  public:

    inline char *GetLastError() {
      return IPWorksZip_Bzip2_GetLastError(m_pObj);
    }
    
    inline int GetLastErrorCode() {
      return IPWorksZip_Bzip2_GetLastErrorCode(m_pObj);
    }

    inline char *VERSION() {
      return (char*)IPWorksZip_Bzip2_Get(m_pObj, 0, 0, 0);
    }

  public: //properties

    inline char* GetArchiveFile() {
      void* val = IPWorksZip_Bzip2_Get(m_pObj, 1, 0, 0);
      return (char*)val;
    }

    inline int SetArchiveFile(const char *lpArchiveFile) {
      return IPWorksZip_Bzip2_Set(m_pObj, 1, 0, (void*)lpArchiveFile, 0);
    }

    inline char* GetExtractToPath() {
      void* val = IPWorksZip_Bzip2_Get(m_pObj, 2, 0, 0);
      return (char*)val;
    }

    inline int SetExtractToPath(const char *lpExtractToPath) {
      return IPWorksZip_Bzip2_Set(m_pObj, 2, 0, (void*)lpExtractToPath, 0);
    }

    inline char* GetFileDecompressedName() {
      void* val = IPWorksZip_Bzip2_Get(m_pObj, 3, 0, 0);
      return (char*)val;
    }

    inline int SetFileDecompressedName(const char *lpFileDecompressedName) {
      return IPWorksZip_Bzip2_Set(m_pObj, 3, 0, (void*)lpFileDecompressedName, 0);
    }

    inline int GetHasMoreData() {
      void* val = IPWorksZip_Bzip2_Get(m_pObj, 4, 0, 0);
      return (int)(long)val;
    }


  public: //methods

    inline int Abort() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksZip_Bzip2_Do(m_pObj, 2, 0, param, cbparam);
      
      
    }
    inline int Append() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksZip_Bzip2_Do(m_pObj, 3, 0, param, cbparam);
      
      
    }
    inline int Compress() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksZip_Bzip2_Do(m_pObj, 4, 0, param, cbparam);
      
      
    }
    inline char* Config(const char* lpszConfigurationString) {
      void *param[1+1] = {(void*)IPZ64CAST(lpszConfigurationString), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksZip_Bzip2_Do(m_pObj, 5, 1, param, cbparam);
      
      return (char*)param[1];
    }
    inline int Extract() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksZip_Bzip2_Do(m_pObj, 6, 0, param, cbparam);
      
      
    }
    inline int ExtractAll() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksZip_Bzip2_Do(m_pObj, 7, 0, param, cbparam);
      
      
    }
    inline int Reset() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksZip_Bzip2_Do(m_pObj, 8, 0, param, cbparam);
      
      
    }
    inline int Scan() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksZip_Bzip2_Do(m_pObj, 9, 0, param, cbparam);
      
      
    }

};


#ifdef WIN32 //UNICODE

typedef struct {
  int Index;
  int Skip;
  int reserved;
} Bzip2BeginFileEventParamsW;

typedef struct {
  int Index;
  int reserved;
} Bzip2EndFileEventParamsW;

typedef struct {
  LPWSTR Description;
  int ErrorCode;
  int Index;
  LPWSTR Filename;
  int Ignore;
  int reserved;
} Bzip2ErrorEventParamsW;

typedef struct {
  LPWSTR Filename;
  int Overwrite;
  int reserved;
} Bzip2OverwriteEventParamsW;

typedef struct {
  LPWSTR Data;
  LPWSTR Filename;
  ns_int64 *pBytesProcessed;
  int PercentProcessed;
  int lenData;
  int reserved;
} Bzip2ProgressEventParamsW;



class Bzip2W : public Bzip2 {

  public: //properties
  
    inline LPWSTR GetArchiveFile() {
      return (LPWSTR)IPWorksZip_Bzip2_Get(m_pObj, 10000+1, 0, 0);
    }

    inline int SetArchiveFile(LPWSTR lpArchiveFile) {
      return IPWorksZip_Bzip2_Set(m_pObj, 10000+1, 0, (void*)lpArchiveFile, 0);
    }

    inline LPWSTR GetExtractToPath() {
      return (LPWSTR)IPWorksZip_Bzip2_Get(m_pObj, 10000+2, 0, 0);
    }

    inline int SetExtractToPath(LPWSTR lpExtractToPath) {
      return IPWorksZip_Bzip2_Set(m_pObj, 10000+2, 0, (void*)lpExtractToPath, 0);
    }

    inline LPWSTR GetFileDecompressedName() {
      return (LPWSTR)IPWorksZip_Bzip2_Get(m_pObj, 10000+3, 0, 0);
    }

    inline int SetFileDecompressedName(LPWSTR lpFileDecompressedName) {
      return IPWorksZip_Bzip2_Set(m_pObj, 10000+3, 0, (void*)lpFileDecompressedName, 0);
    }





  public: //events
  
    virtual int FireBeginFile(Bzip2BeginFileEventParamsW *e) {return 0;}
    virtual int FireEndFile(Bzip2EndFileEventParamsW *e) {return 0;}
    virtual int FireError(Bzip2ErrorEventParamsW *e) {return 0;}
    virtual int FireOverwrite(Bzip2OverwriteEventParamsW *e) {return 0;}
    virtual int FireProgress(Bzip2ProgressEventParamsW *e) {return 0;}


  protected:
  
    virtual int Bzip2EventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {
    	int ret_code = 0;
      switch (event_id) {
         case 1: {
            Bzip2BeginFileEventParamsW e = {(int)IPZ64CAST(param[0]), (int)IPZ64CAST(param[1]),  0};
            ret_code = FireBeginFile(&e);
            param[1] = (void*)(e.Skip);
            break;
         }
         case 2: {
            Bzip2EndFileEventParamsW e = {(int)IPZ64CAST(param[0]),  0};
            ret_code = FireEndFile(&e);
            break;
         }
         case 3: {
            Bzip2ErrorEventParamsW e = {(LPWSTR)IPZ64CAST(param[0]), (int)IPZ64CAST(param[1]), (int)IPZ64CAST(param[2]), (LPWSTR)IPZ64CAST(param[3]), (int)IPZ64CAST(param[4]),  0};
            ret_code = FireError(&e);
            param[4] = (void*)(e.Ignore);
            break;
         }
         case 4: {
            Bzip2OverwriteEventParamsW e = {(LPWSTR)IPZ64CAST(param[0]), (int)IPZ64CAST(param[1]),  0};
            ret_code = FireOverwrite(&e);
            param[0] = (void*)(e.Filename);
            param[1] = (void*)(e.Overwrite);
            break;
         }
         case 5: {
            Bzip2ProgressEventParamsW e = {(LPWSTR)IPZ64CAST(param[0]), (LPWSTR)IPZ64CAST(param[1]), (ns_int64*)IPZ64CAST(param[2]), (int)IPZ64CAST(param[3]), (int)IPZ64CAST(cbparam[0]),  0};
            ret_code = FireProgress(&e);
            break;
         }

      }
      return ret_code;
    }
  
  public: //event overrides

    virtual int FireBeginFile(Bzip2BeginFileEventParams *e) {return -10000;}
    virtual int FireEndFile(Bzip2EndFileEventParams *e) {return -10000;}
    virtual int FireError(Bzip2ErrorEventParams *e) {return -10000;}
    virtual int FireOverwrite(Bzip2OverwriteEventParams *e) {return -10000;}
    virtual int FireProgress(Bzip2ProgressEventParams *e) {return -10000;}

  public: //methods

    inline int Abort() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksZip_Bzip2_Do(m_pObj, 10000+2, 0, param, cbparam);
      
    }
    inline int Append() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksZip_Bzip2_Do(m_pObj, 10000+3, 0, param, cbparam);
      
    }
    inline int Compress() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksZip_Bzip2_Do(m_pObj, 10000+4, 0, param, cbparam);
      
    }
    inline LPWSTR Config(LPWSTR lpszConfigurationString) {
      void *param[1+1] = {(void*)lpszConfigurationString, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksZip_Bzip2_Do(m_pObj, 10000+5, 1, param, cbparam);
      return (LPWSTR)param[1];
    }
    inline int Extract() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksZip_Bzip2_Do(m_pObj, 10000+6, 0, param, cbparam);
      
    }
    inline int ExtractAll() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksZip_Bzip2_Do(m_pObj, 10000+7, 0, param, cbparam);
      
    }
    inline int Reset() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksZip_Bzip2_Do(m_pObj, 10000+8, 0, param, cbparam);
      
    }
    inline int Scan() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksZip_Bzip2_Do(m_pObj, 10000+9, 0, param, cbparam);
      
    }

};

#endif //WIN32

#endif //_IPWORKSZIP_BZIP2_H_




